﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using MVVMbasics.Services;
using Newtonsoft.Json;
using SimpleChat.Models;

namespace SimpleChat.Services
{
	public class BackendService : IService
	{
		private const string ServerAddress = "http://localhost:2939/Chat/";

		private DateTime _lastUpdated = new DateTime();

		public async Task<int> LoginAsync(string username)
		{
			using (HttpClient client = new HttpClient())
			{
				string request = String.Format("{0}Login?username={1}", ServerAddress, username);
				using (HttpResponseMessage response = await client.GetAsync(request))
				{
					using (HttpContent content = response.Content)
					{
						string jsonResult = await content.ReadAsStringAsync();
						int user = await JsonConvert.DeserializeObjectAsync<int>(jsonResult);
						return user;
					}
				}
			}
		}

		public async void SendMessageAsync(string message, int userId)
		{
			using (HttpClient client = new HttpClient())
			{
				string request = String.Format("{0}NewMessage?userId={1}&content={2}", ServerAddress, userId.ToString(), message);
				using (HttpResponseMessage response = await client.GetAsync(request))
				{
					if (response.StatusCode != HttpStatusCode.OK)
						throw new Exception("Error while sending");
				}
			}
		}

		public async Task<List<MessageModel>> GetNewMessagesAsync()
		{
			using (HttpClient client = new HttpClient())
			{
				string request = String.Format("{0}GetNewMessages?since={1}", ServerAddress, _lastUpdated.ToString("yyyyMMddHHmmss"));
				_lastUpdated = DateTime.Now;
				using (HttpResponseMessage response = await client.GetAsync(request))
				{
					using (HttpContent content = response.Content)
					{
						string jsonResult = await content.ReadAsStringAsync();
						List<MessageModel> messages = await JsonConvert.DeserializeObjectAsync<List<MessageModel>>(jsonResult);
						return messages;
					}
				}
			}
		}
	}
}
