﻿using MVVMbasics.Commands;
using MVVMbasics.Viewmodels;
using MVVMbasics.Services;
using SimpleChat.Models;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using MVVMbasics.Attributes;
using SimpleChat.Services;

namespace SimpleChat.Viewmodels
{
	[MvvmCommandAutobinding]
	public class ChatViewmodel : BaseViewmodel
	{
		private BackendService _backend;
		private ITimerService _timer;
		private Guid _refreshTimerId;
		private int _userId;

		private ObservableCollection<MessageModel> _messageHistory = new ObservableCollection<MessageModel>();
		public ObservableCollection<MessageModel> MessageHistory
		{
			get { return _messageHistory; }
			set { Set(ref _messageHistory, value); }
		}

		private string _newMessage;
		public string NewMessage
		{
			get { return _newMessage; }
			set { Set(ref _newMessage, value); }
		}

		public BaseCommand SendCommand { get; set; }
		public BaseCommand RefreshCommand { get; set; }

		public override void OnServiceLocatorAvailable()
		{
			base.OnServiceLocatorAvailable();

			_backend = ServiceLocator.Retrieve<BackendService>();
			_timer = ServiceLocator.Retrieve<ITimerService>();
		}

		public override void OnNavigatedTo(ParameterList uriParameters, ParameterList parameters, ViewState viewState)
		{
			base.OnNavigatedTo(uriParameters, parameters, viewState);

			if (viewState == ViewState.Activated)
			{
				parameters.TryGetInt("user", out _userId);

				ServiceLocator.Retrieve<INavigatorService>().RemoveBackEntry();
			}

			// Load the full list of messages
			Refresh();

			// Start a new timer that checks for new messages regularly and displays them
			_refreshTimerId = _timer.StartLooping(TimeSpan.FromSeconds(10), Refresh);
		}

		public override void OnNavigatedFrom(ViewState viewState)
		{
			base.OnNavigatedFrom(viewState);

			if (viewState == ViewState.Hidden)
			{
				// If a timer that checks for notifications is active, stop it
				_timer.Stop(_refreshTimerId);
			}
		}

		private void Send()
		{
			if (!String.IsNullOrWhiteSpace(NewMessage))
			{
				try
				{
					_backend.SendMessageAsync(NewMessage, _userId);
				}
				catch
				{
					ServiceLocator.RetrieveOnce<IMessageboxService>().Show("Fehler beim Senden der Nachricht!");
					return;
				}
				NewMessage = String.Empty;
			}
		}

		private async void Refresh()
		{
			List<MessageModel> newMessages;
			try
			{
				newMessages = await _backend.GetNewMessagesAsync();
			}
			catch
			{
				ServiceLocator.RetrieveOnce<IMessageboxService>().Show("Fehler beim Aktualisieren der Nachrichten!");
				return;
			}
			foreach (var message in newMessages)
				MessageHistory.Insert(0, message);
		}
	}
}
